using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace CheesyFX
{
    public static class AudioManager
    {
        public static float baseVolume = 1f;
        public static float basePitch = 1f;
        private static List<Atom> soundProducers;
        private static List<AudioClip> _clips;
        public static List<AudioClip> clips
        {
            get { return _clips;}
            set
            {
                _clips = value;
                remainingClips = new List<AudioClip>(clips);
            }
        }
        private static List<AudioClip> remainingClips;
        private static System.Random rnd = new System.Random();

        private static AudioClip GetRandomClip()
        {
            AudioClip clip;
            if(clips.Count == 1) clip = clips[0];
            else{
                int id = rnd.Next(0, remainingClips.Count);
                clip = remainingClips[id];
                remainingClips.Remove(clip);
                if(remainingClips.Count == 0) remainingClips = new List<AudioClip>(clips);
            }
            // clip.name.Print();
            return clip;
        }

        public static void Play(Collision collision, AudioSource audioSource, BoxCollider collider)
        {
            if (soundProducers != null && !soundProducers.Contains(collision.collider.GetAtom())) return;
            var transform = collider.transform;
            var localScale = transform.localScale;
            var area = localScale.x * localScale.z * 10000f;
            area = Mathf.Pow(area, .25f);
            var volume = baseVolume * Mathf.Abs(Vector3.Dot(collision.relativeVelocity, collision.contacts[0].normal) / area);
            volume *= Mathf.Lerp(.3f, 1.4f, (localScale.y - .01f)*5.2f);
            var pitch = basePitch * Mathf.Lerp(1.3f, .8f, (area - 1f)*.5f);
            // audioSource.pitch.Print();
            var directionFactor = Mathf.Abs(transform.InverseTransformDirection(collision.relativeVelocity).normalized.y);
            // if(directionFactor < 0f) return;
            volume *= directionFactor * directionFactor * directionFactor;
            pitch *= Mathf.Pow(directionFactor, .25f);
            // pitch.Print();
            audioSource.pitch = pitch;
            audioSource.PlayOneShot(GetRandomClip(), volume);
        }

        public static void SetSoundProducers(string choice, bool onlyThis)
        {
            if (onlyThis)
            {
                if (choice == null) soundProducers = null;
                else soundProducers = new List<Atom>{SuperController.singleton.GetAtomByUid(choice)};
                return;
            }
            string[] types = new string[]{};
            switch (choice)
            {
                case "Everything":
                {
                    soundProducers = null;
                    HeelAdjust2.onlySoundFromThis.popup.gameObject.SetActive(false);
                    return;
                }
                case "Floors, Walls, Glass, CUAs":
                {
                    types = new []{"Panel", "Slate", "Wall", "Glass", "CustomUnityAsset" };
                    break;
                }
                case "Floors, Walls, Glass":
                {
                    types = new []{"Panel", "Slate", "Wall", "Glass" };
                    break;
                }
                case "CUAs":
                {
                    types = new []{"CustomUnityAsset" };
                    break;
                }
                case "Only this...":
                {
                    HeelAdjust2.onlySoundFromThis.popup.gameObject.SetActive(true);
                    return;
                }
            }
            soundProducers = SuperController.singleton.GetAtoms().Where(x => types.Any(t => x.type.Contains(t))).ToList();
            HeelAdjust2.onlySoundFromThis.popup.gameObject.SetActive(false);
        }
        
        public static Atom GetAtom(this Collider collider)
        {
            Atom atom = null;
            Transform parent = collider.attachedRigidbody.transform.parent;
            int i = 0;
            while (atom == null && i < 20)
            {
                atom = parent.gameObject.GetComponent<Atom>();
                parent = parent.parent;
                i += 1;
            }
            return atom;
        }
    }
}