using System.Collections;
using UnityEngine;

namespace CheesyFX
{
    public class CollisionHandler : MonoBehaviour
    {
        public TriggerHandler triggerHandler;
        public AudioSource audioSource;
        private int collisions;
        private bool isOnStay;
        private IEnumerator onStayRoutine;
        private static float onStayTimeOut = .1f;
        private float onStayTimer;
        
        public void OnCollisionEnter(Collision collision){
            if(!enabled) return;
            if(!isOnStay){
                collisions++;
                var collider = triggerHandler != null && !triggerHandler.isTriggered
                    ? HeelAdjust2.lHeelCollider
                    : HeelAdjust2.lToeCollider;
                AudioManager.Play(collision, audioSource, collider);
                // isOnStay = true;
                onStayRoutine = OnStayRoutine();
                StartCoroutine(onStayRoutine);
            }
            onStayTimer = onStayTimeOut;
        }
        
        public void OnCollisionExit(Collision collision)
        {
            if(collisions > 0) collisions--;
        }
        
        public IEnumerator OnStayRoutine(){
            while(onStayTimer > 0f)
            {
                if(collisions == 0) onStayTimer -= Time.deltaTime;
                yield return new WaitForSeconds(Time.deltaTime);
            }
            isOnStay = false;
        }
    }

    public class TriggerHandler : MonoBehaviour
    {
        public bool isTriggered;
        private void OnTriggerEnter(Collider other)
        {
            isTriggered = true;
        }

        private void OnTriggerExit(Collider other)
        {
            isTriggered = false;
        }
    }
}